class TinySA():

    def __init__(self):
        self.suffixes = []
        self.positions = []

    def index(self, text):
        self._get_suffixes_of(text)
        self._sort_suffixes_between(0, len(text) - 1)

    def _get_suffixes_of(self, text):
        for i in range(len(text)):
            self.suffixes.append(text[i:])
            self.positions.append(i)

    def _sort_suffixes_between(self, i, j):
        p_suffix = self.suffixes[i]  # pivot suffix
        p_position = self.positions[i]  # pivot position
        l_suffixes = []  # left suffixes
        r_suffixes = []  # right suffixes
        l_positions = []  # left positions
        r_positions = []  # right positions
        for x in range(i + 1, j + 1):
            if self.suffixes[x] <= p_suffix:
                l_suffixes.append(self.suffixes[x])
                l_positions.append(self.positions[x])
            else:
                r_suffixes.append(self.suffixes[x])
                r_positions.append(self.positions[x])
        self.suffixes[i:j+1] = l_suffixes + [p_suffix] + r_suffixes
        self.positions[i:j+1] = l_positions + [p_position] + r_positions
        if len(l_suffixes) > 1:
            self._sort_suffixes_between(i, len(l_suffixes) - 1)
        if len(r_suffixes) > 1:
            self._sort_suffixes_between(len(l_suffixes) + 1, j)

    def search(self, string):
        begin = 0
        end = len(self.suffixes)
        while begin < end:
            half = (begin + end) // 2
            suffix = self.suffixes[half]
            if suffix < string:
                begin = half + 1
            elif suffix > string:
                end = half
            else:
                return half
        return -1



import unittest
from tinysa.tinysa import TinySA


class TestTinySA(unittest.TestCase):

    def test_index(self):
        suffix_array = TinySA()
        suffix_array.index('banana')
        self.assertEqual(suffix_array.positions, [5, 3, 1, 0, 4, 2])

    def test__get_suffixes(self):
        suffix_array = TinySA()
        suffix_array._get_suffixes_of('banana')
        suffixes = ['banana', 'anana', 'nana', 'ana', 'na', 'a']
        self.assertEqual(suffix_array.suffixes, suffixes)

    def test__sort_suffixes_between(self):
        suffix_array = TinySA()
        suffix_array._get_suffixes_of('banana')
        suffix_array._sort_suffixes_between(0, len(suffix_array.suffixes) - 1)
        sorted_suffixes = ['a', 'ana', 'anana', 'banana', 'na', 'nana']
        sorted_positions = [5, 3, 1, 0, 4, 2]
        self.assertEqual(suffix_array.suffixes, sorted_suffixes)
        self.assertEqual(suffix_array.positions, sorted_positions)

    def test_search(self):
        suffix_array = TinySA()
        suffix_array.index('banana')
        self.assertEqual(suffix_array.search('ana'), 1)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

setup(
    author="Jun Harashima",
    author_email='j.harashima@gmail.com',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    description="Python Boilerplate contains all the boilerplate you need to create a Python package.",
    license="MIT license",
    long_description=readme,
    include_package_data=True,
    keywords='tiny suffix array',
    name='tinysa',
    packages=find_packages(include=['tinysa']),
    test_suite='tests',
    url='https://github.com/jun-harashima/tinysa',
    version='0.2.1',
    zip_safe=False,
)
